"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_uniq_1 = __importDefault(require("lodash.uniq"));
const common_1 = require("@affinidi/common");
const tools_common_1 = require("@affinidi/tools-common");
const _helpers_1 = require("../_helpers");
const SdkErrorFromCode_1 = __importDefault(require("../shared/SdkErrorFromCode"));
let HolderService = class HolderService {
    constructor({ registryUrl, metricsUrl, accessApiKey, keysService, keyManager }, platformCryptographyTools, component, resolveLegacyElemLocally, beforeDocumentLoader) {
        this._didMap = {};
        this._affinityService = new common_1.Affinity({
            apiKey: accessApiKey,
            registryUrl,
            metricsUrl,
            component,
            resolveLegacyElemLocally,
            beforeDocumentLoader,
            keysService,
            keyManager,
        }, platformCryptographyTools);
        this._digestService = new common_1.DigestService();
    }
    async buildCredentialOfferResponse(credentialOfferToken) {
        const credentialOffer = common_1.JwtService.fromJWT(credentialOfferToken);
        const { interactionToken: offerRequestInteractionToken } = credentialOffer.payload;
        const { callbackURL, offeredCredentials } = offerRequestInteractionToken;
        const selectedCredentials = offeredCredentials;
        const interactionToken = {
            callbackURL,
            selectedCredentials,
        };
        const offerResponse = await common_1.JwtService.buildJWTInteractionToken(interactionToken, 'credentialOfferResponse', credentialOffer);
        return offerResponse;
    }
    async buildCredentialResponse(credentialRequestToken, suppliedCredentials, expiresAt) {
        const credentialRequest = common_1.JwtService.fromJWT(credentialRequestToken);
        const { callbackURL } = credentialRequest.payload.interactionToken;
        const interactionToken = {
            callbackURL,
            suppliedCredentials,
        };
        const credentialResponse = await common_1.JwtService.buildJWTInteractionToken(interactionToken, 'credentialResponse', credentialRequest, expiresAt);
        return credentialResponse;
    }
    /* istanbul ignore next: private method */
    _keyIdToDid(keyId) {
        return common_1.DidDocumentService.keyIdToDid(keyId);
    }
    /* istanbul ignore next: private method */
    async _resolveUniqDIDs(dids) {
        const promises = [];
        const uniqDIDs = (0, lodash_uniq_1.default)(dids);
        for (const did of uniqDIDs) {
            promises.push(this._resolveDid(did));
        }
        await Promise.all(promises);
    }
    /* istanbul ignore next: private method */
    async _resolveDid(did) {
        const didDocument = await this._affinityService.resolveDid(did);
        this._didMap[did] = didDocument;
        return didDocument;
    }
    /* istanbul ignore next: private method */
    async _validateCredentials(credentials, holderKey) {
        const signatureValidationResults = [];
        for (const credential of credentials) {
            const didDocument = this._didMap[this._keyIdToDid(credential.issuer)];
            const { result, error } = await this._affinityService.validateCredential(credential, holderKey, didDocument);
            signatureValidationResults.push({ result, error });
        }
        return signatureValidationResults;
    }
    async verifyCredentialShareResponse(credentialShareResponseToken, credentialShareRequestToken, 
    /* istanbul ignore next: shouldOwn = true is covered ! */
    shouldOwn = true) {
        let isValid = true;
        const errors = [];
        const credentialShareResponse = common_1.JwtService.fromJWT(credentialShareResponseToken);
        const { iss: issuer, jti } = credentialShareResponse.payload;
        const { suppliedCredentials } = credentialShareResponse.payload.interactionToken;
        const didArray = [];
        const holderDid = this._keyIdToDid(issuer);
        didArray.push(holderDid);
        for (const credential of suppliedCredentials) {
            const issuerDid = this._keyIdToDid(credential.issuer);
            didArray.push(issuerDid);
        }
        await this._resolveUniqDIDs(didArray);
        const didDocument = this._didMap[holderDid];
        try {
            await this._affinityService.validateJWT(credentialShareResponseToken, credentialShareRequestToken, didDocument);
        }
        catch (error) {
            if (error.message === 'Token expired') {
                throw new SdkErrorFromCode_1.default('COR-19');
            }
            if (error.message === 'Invalid Token') {
                throw new SdkErrorFromCode_1.default('COR-35');
            }
            throw error;
        }
        let results;
        if (shouldOwn) {
            results = await this._validateCredentials(suppliedCredentials, issuer);
        }
        else {
            results = await this._validateCredentials(suppliedCredentials);
        }
        for (const result of results) {
            if (result.result === false) {
                isValid = false;
                errors.push(result.error);
            }
        }
        return { isValid, did: holderDid, jti, suppliedCredentials, errors };
    }
    /**
     * @description Slightly modified version of Affinity.validateJWT,
     * this validates that the given challenge was signed by the expected
     * issuer and that it isn't exipred.
     * @param vp - the presentation to be validated
     * when needed to verify if holder is a subject of VC
     * @returns { isValid, did, challenge, suppliedPresentations, errors }
     *
     * isValid - boolean, result of the verification
     *
     * did - DID of the VP issuer (holder of the shared VCs)
     *
     * challenge - unique identifier for the presentation.
     * You are responsible for checking this to protect against replay attacks
     *
     * suppliedPresentations - the validated presentation
     *
     * errors - array of validation errors
     */
    async verifyPresentationChallenge(challenge, expectedIssuer) {
        const token = common_1.Affinity.fromJwt(challenge);
        const { payload } = token;
        const strippedExpectedIssuer = (0, _helpers_1.stripParamsFromDidUrl)(expectedIssuer);
        const strippedPayloadIssuer = (0, _helpers_1.stripParamsFromDidUrl)(payload.iss);
        if (strippedExpectedIssuer !== strippedPayloadIssuer) {
            throw new Error('Token not issued by expected issuer.');
        }
        const did = common_1.DidDocumentService.keyIdToDid(expectedIssuer);
        const didDocument = await this._affinityService.resolveDid(did);
        const publicKey = common_1.DidDocumentService.getPublicKey(strippedExpectedIssuer, didDocument, payload.kid);
        const { digest: tokenDigest, signature } = this._digestService.getTokenDigest(token);
        const isSignatureVerified = common_1.KeysService.verify(tokenDigest, publicKey, signature);
        if (!isSignatureVerified) {
            throw new Error('Signature on token is invalid');
        }
        if (payload.exp < Date.now()) {
            throw new Error('Token expired');
        }
    }
    async verifyCredentialOfferRequest(credentialOfferRequestToken) {
        try {
            await this._affinityService.validateJWT(credentialOfferRequestToken);
        }
        catch (error) {
            if (error.message === 'Token expired') {
                return { isValid: false, errorCode: 'COR-19', error: error.message };
            }
            if (error.message === 'Invalid Token') {
                return { isValid: false, errorCode: 'COR-35', error: error.message };
            }
            if (error.message === 'Signature on token is invalid') {
                return { isValid: false, errorCode: 'COR-28', error: error.message };
            }
            return { isValid: false, errorCode: '', error: error.message };
        }
        return { isValid: true, error: '', errorCode: '' };
    }
};
HolderService = __decorate([
    (0, tools_common_1.profile)()
], HolderService);
exports.default = HolderService;
//# sourceMappingURL=HolderService.js.map